import { __awaiter, __decorate } from "tslib";
import { attr, customElement, DOM, FASTElement, html, nullableNumberConverter, observable, Updates } from '@microsoft/fast-element';
import React from "react";
import ReactDOM from "react-dom";
import { uniqueElementName } from '@microsoft/fast-element/testing.js';
import { expect } from "chai";
import { reactWrapper } from './index.js';
const restTestEvents = {
    onFoo: 'foo',
    onBar: 'bar',
};
function emitEvent(element, type) {
    const e = new CustomEvent(type);
    element.dispatchEvent(e);
}
const vanillaElementName = uniqueElementName();
class VanillaElement extends HTMLElement {
    constructor() {
        super(...arguments);
        this.bool = false;
        this.str = '';
        this.num = -1;
        this.obj = null;
        this.arr = null;
        this._rstr = '';
        this._rbool = false;
        this._rnum = -1;
    }
    get rstr() {
        return this._rstr;
    }
    set rstr(value) {
        this._rstr = value;
        this.setAttribute('rstr', value.toString());
    }
    get rbool() {
        return this._rbool;
    }
    set rbool(value) {
        this._rbool = value;
        DOM.setBooleanAttribute(this, 'rbool', value);
    }
    get rnum() {
        return this._rnum;
    }
    set rnum(value) {
        this._rnum = value;
        this.setAttribute('rnum', value.toString());
    }
    attributeChangedCallback(name, oldValue, newValue) {
        switch (name) {
            case 'rstr':
                this._rstr = newValue;
                break;
            case 'rbool':
                if (newValue === '') {
                    this._rbool = true;
                }
                else {
                    this._rbool = false;
                }
                break;
            case 'rnum':
                this._rnum = nullableNumberConverter.fromView(newValue);
                break;
        }
    }
}
customElements.define(vanillaElementName, VanillaElement);
const decoratedElementName = uniqueElementName();
let DecoratedTestElement = class DecoratedTestElement extends FASTElement {
    constructor() {
        super(...arguments);
        this.bool = false;
        this.str = '';
        this.num = -1;
        this.obj = null;
        this.arr = null;
        this.rbool = false;
        this.rstr = '';
        this.rnum = -1;
    }
};
__decorate([
    observable
], DecoratedTestElement.prototype, "bool", void 0);
__decorate([
    observable
], DecoratedTestElement.prototype, "str", void 0);
__decorate([
    observable
], DecoratedTestElement.prototype, "num", void 0);
__decorate([
    observable
], DecoratedTestElement.prototype, "obj", void 0);
__decorate([
    observable
], DecoratedTestElement.prototype, "arr", void 0);
__decorate([
    attr({ mode: "boolean" })
], DecoratedTestElement.prototype, "rbool", void 0);
__decorate([
    attr
], DecoratedTestElement.prototype, "rstr", void 0);
__decorate([
    attr({ converter: nullableNumberConverter })
], DecoratedTestElement.prototype, "rnum", void 0);
DecoratedTestElement = __decorate([
    customElement({
        name: decoratedElementName,
        template: html `<slot></slot>`
    })
], DecoratedTestElement);
class DefinitionTestElement extends FASTElement {
    constructor() {
        super(...arguments);
        this.bool = false;
        this.str = '';
        this.num = -1;
        this.obj = null;
        this.arr = null;
        this.rbool = false;
        this.rstr = '';
        this.rnum = -1;
    }
}
__decorate([
    observable
], DefinitionTestElement.prototype, "bool", void 0);
__decorate([
    observable
], DefinitionTestElement.prototype, "str", void 0);
__decorate([
    observable
], DefinitionTestElement.prototype, "num", void 0);
__decorate([
    observable
], DefinitionTestElement.prototype, "obj", void 0);
__decorate([
    observable
], DefinitionTestElement.prototype, "arr", void 0);
__decorate([
    attr({ mode: "boolean" })
], DefinitionTestElement.prototype, "rbool", void 0);
__decorate([
    attr
], DefinitionTestElement.prototype, "rstr", void 0);
__decorate([
    attr({ converter: nullableNumberConverter })
], DefinitionTestElement.prototype, "rnum", void 0);
const def = DefinitionTestElement.compose({
    name: uniqueElementName(),
    template: html `<slot></slot>`
});
const scenarios = [
    {
        description: 'Wrapping a decorated FASTElement',
        elementName: decoratedElementName,
        wrap: (x) => x(DecoratedTestElement, {
            events: restTestEvents
        })
    },
    {
        description: 'Wrapping a FASTElementDefinition',
        elementName: def.name,
        wrap: (x) => x(def, {
            events: restTestEvents
        })
    },
    {
        description: 'Wrapping a vanilla Web Component',
        elementName: vanillaElementName,
        wrap: (x) => x(VanillaElement, {
            name: vanillaElementName,
            events: restTestEvents,
            properties: [
                'bool',
                'str',
                'num',
                'obj',
                'arr',
                'rbool',
                'rstr',
                'rnum'
            ]
        })
    }
];
for (const scenario of scenarios) {
    describe(scenario.description, () => {
        let container;
        beforeEach(() => {
            container = document.createElement('div');
            document.body.appendChild(container);
        });
        afterEach(() => {
            if (container && container.parentNode) {
                container.parentNode.removeChild(container);
            }
        });
        const wrap = reactWrapper(React);
        const WrappedComponent = scenario.wrap(wrap);
        let el;
        const renderReactComponent = (
        /* eslint-disable-next-line @typescript-eslint/ban-types */
        props) => {
            ReactDOM.render(React.createElement(WrappedComponent, Object.assign({}, props)), container);
            el = container.querySelector(scenario.elementName);
        };
        it('works with text children', () => __awaiter(void 0, void 0, void 0, function* () {
            const name = 'World';
            ReactDOM.render(React.createElement(WrappedComponent, null,
                "Hello ",
                name), container);
            el = container.querySelector(scenario.elementName);
            expect(el.textContent).to.equal('Hello World');
        }));
        it('does not recreate the same component twice', () => __awaiter(void 0, void 0, void 0, function* () {
            const SameWrappedComponent = scenario.wrap(wrap);
            expect(WrappedComponent).to.eq(SameWrappedComponent);
        }));
        it('wrapper renders custom element that updates', () => __awaiter(void 0, void 0, void 0, function* () {
            renderReactComponent();
            expect(el).ok;
            expect(el.isConnected).ok;
        }));
        it('can get ref to element', () => __awaiter(void 0, void 0, void 0, function* () {
            const elementRef1 = React.createRef();
            renderReactComponent({ ref: elementRef1 });
            expect(elementRef1.current).equal(el);
            const elementRef2 = React.createRef();
            renderReactComponent({ ref: elementRef2 });
            expect(elementRef1.current).equal(null);
            expect(elementRef2.current).equal(el);
            renderReactComponent({ ref: elementRef1 });
            expect(elementRef1.current).equal(el);
            expect(elementRef2.current).equal(null);
        }));
        it('can get ref to element via callbacks', () => __awaiter(void 0, void 0, void 0, function* () {
            const ref1Calls = [];
            const refCb1 = (e) => ref1Calls.push(e === null || e === void 0 ? void 0 : e.localName);
            const ref2Calls = [];
            const refCb2 = (e) => ref2Calls.push(e === null || e === void 0 ? void 0 : e.localName);
            renderReactComponent({ ref: refCb1 });
            expect(ref1Calls).eql([scenario.elementName]);
            renderReactComponent({ ref: refCb2 });
            expect(ref1Calls).eql([scenario.elementName, undefined]);
            expect(ref2Calls).eql([scenario.elementName]);
            renderReactComponent({ ref: refCb1 });
            expect(ref1Calls).eql([scenario.elementName, undefined, scenario.elementName]);
            expect(ref2Calls).eql([scenario.elementName, undefined]);
        }));
        it('can set attributes', () => __awaiter(void 0, void 0, void 0, function* () {
            yield renderReactComponent({ id: 'id' });
            expect(el.getAttribute('id')).equal('id');
            yield renderReactComponent({ id: undefined });
            expect(el.getAttribute('id')).equal(null);
            yield renderReactComponent({ id: 'id2' });
            expect(el.getAttribute('id')).equal('id2');
        }));
        it('can set properties', () => __awaiter(void 0, void 0, void 0, function* () {
            let o = { foo: true };
            let a = [1, 2, 3];
            yield renderReactComponent({
                bool: true,
                str: 'str',
                num: 5,
                obj: o,
                arr: a,
            });
            expect(el.bool).equal(true);
            expect(el.str).equal('str');
            expect(el.num).equal(5);
            expect(el.obj).eql(o);
            expect(el.arr).eql(a);
            const firstEl = el;
            // update
            o = { foo: false };
            a = [1, 2, 3, 4];
            yield renderReactComponent({
                bool: false,
                str: 'str2',
                num: 10,
                obj: o,
                arr: a,
            });
            expect(firstEl).equal(el);
            expect(el.bool).equal(false);
            expect(el.str).equal('str2');
            expect(el.num).equal(10);
            expect(el.obj).eql(o);
            expect(el.arr).eql(a);
        }));
        it('can set properties that reflect', () => __awaiter(void 0, void 0, void 0, function* () {
            renderReactComponent({
                rbool: true,
                rstr: 'str',
                rnum: 5,
            });
            yield Updates.next();
            const firstEl = el;
            expect(el.rbool).equal(true);
            expect(el.rstr).equal('str');
            expect(el.rnum).equal(5);
            expect(el.getAttribute('rbool')).equal('');
            expect(el.getAttribute('rstr')).equal('str');
            expect(el.getAttribute('rnum')).equal('5');
            // update
            renderReactComponent({
                rbool: false,
                rstr: 'str2',
                rnum: 10,
            });
            yield Updates.next();
            expect(firstEl).equal(el);
            expect(el.rbool).equal(false);
            expect(el.rstr).equal('str2');
            expect(el.rnum).equal(10);
            expect(el.getAttribute('rbool')).equal(null);
            expect(el.getAttribute('rstr')).equal('str2');
            expect(el.getAttribute('rnum')).equal('10');
        }));
        it('can listen to events', () => __awaiter(void 0, void 0, void 0, function* () {
            let fooEvent, fooEvent2, barEvent;
            const onFoo = (e) => {
                fooEvent = e;
            };
            const onFoo2 = (e) => {
                fooEvent2 = e;
            };
            const onBar = (e) => {
                barEvent = e;
            };
            yield renderReactComponent({
                onFoo,
                onBar,
            });
            emitEvent(el, 'foo');
            expect(fooEvent.type).equal('foo');
            emitEvent(el, 'bar');
            expect(barEvent.type).equal('bar');
            fooEvent = undefined;
            barEvent = undefined;
            yield renderReactComponent({
                onFoo: undefined,
            });
            emitEvent(el, 'foo');
            expect(fooEvent).equal(undefined);
            emitEvent(el, 'bar');
            expect(barEvent.type).equal('bar');
            fooEvent = undefined;
            barEvent = undefined;
            yield renderReactComponent({
                onFoo,
            });
            emitEvent(el, 'foo');
            expect(fooEvent.type).equal('foo');
            emitEvent(el, 'bar');
            expect(barEvent.type).equal('bar');
            yield renderReactComponent({
                onFoo: onFoo2,
            });
            fooEvent = undefined;
            fooEvent2 = undefined;
            emitEvent(el, 'foo');
            expect(fooEvent).equal(undefined);
            expect(fooEvent2.type).equal('foo');
            yield renderReactComponent({
                onFoo,
            });
            fooEvent = undefined;
            fooEvent2 = undefined;
            emitEvent(el, 'foo');
            expect(fooEvent.type).equal('foo');
            expect(fooEvent2).equal(undefined);
        }));
        it('can listen to native events', () => __awaiter(void 0, void 0, void 0, function* () {
            let clickEvent;
            yield renderReactComponent({
                onClick(e) {
                    e.persist();
                    clickEvent = e;
                },
            });
            el.click();
            expect(clickEvent === null || clickEvent === void 0 ? void 0 : clickEvent.type).equal('click');
        }));
        it('can set children', () => __awaiter(void 0, void 0, void 0, function* () {
            const children = React.createElement('div'
            // Note, constructing children like this is rare and the React type expects
            // this to be an HTMLCollection even though that's not the output of
            // `createElement`.
            );
            yield renderReactComponent({ children });
            expect(el.childNodes.length).equal(1);
            expect(el.firstElementChild.localName).equal('div');
        }));
        it('can set reserved React properties', () => __awaiter(void 0, void 0, void 0, function* () {
            yield renderReactComponent({
                style: { display: 'block' },
                className: 'foo bar',
            });
            expect(el.style.display).equal('block');
            expect(el.getAttribute('class')).equal('foo bar');
        }));
        it('warns if element contains reserved props', () => __awaiter(void 0, void 0, void 0, function* () {
            const warn = console.warn;
            let warning;
            console.warn = (m) => {
                warning = m;
            };
            let Warn = class Warn extends FASTElement {
                constructor() {
                    super(...arguments);
                    this.ref = 'hi';
                }
            };
            __decorate([
                attr
            ], Warn.prototype, "ref", void 0);
            Warn = __decorate([
                customElement(uniqueElementName())
            ], Warn);
            const wrap = reactWrapper(React);
            const WarningComponent = wrap(Warn);
            ReactDOM.render(React.createElement(WarningComponent, null), container);
            expect(warning).include('ref');
            console.warn = warn;
        }));
    });
}
/* eslint-enable @typescript-eslint/no-non-null-assertion */
